extends Resource
class_name DragonOutfit


@export var allow_null: bool
@export var outfits: Array[Texture2D]
var index: int = 0


func _init(outfits: Array[Texture2D] = []):
	self.outfits = outfits


func pick_next() -> Texture2D:
	index += 1
	if allow_null:
		return _pick_nullable()
	else:
		return _pick_non_nullable()


func pick_previous() -> Texture2D:
	index -= 1
	if allow_null:
		return _pick_nullable()
	else:
		return _pick_non_nullable()


func _pick_nullable():
	index %= (len(outfits) + 1)
	if index == 0:
		return null
	
	return outfits[index - 1]


func _pick_non_nullable():
	index %= len(outfits)
	return outfits[index]


func get_index(texture: Texture2D) -> int:
	return outfits.find(texture)


func get_texture(index: int) -> Texture2D:
	if index < 0 or index >= len(outfits):
		return null
	
	return outfits[index]
